"""
MAUS web application image utilities.
"""

#  This file is part of MAUS: http://micewww.pp.rl.ac.uk:8080/projects/maus
#
#  MAUS is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  MAUS is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with MAUS.  If not, see <http://www.gnu.org/licenses/>.

import os.path
import Image

from django.core.files.storage import default_storage
from django.http import Http404
from django.http import HttpResponse

from magickwand.image import Image as MagickWandImage # pylint: disable=W0404

IMAGE_TYPES = ["ps", "eps", "png", "pdf", "gif", "jpg", "jpeg"]

def parse_path(file_name):
    """
    Given a file name of form /path.../file.ext split it into
    the path to the file name and the extension i.e.
    [/path.../file, ext]
    @param file_name File name.
    @return path and extension.
    """
    [file_name, file_ext] = os.path.splitext(file_name)
    return [file_name, file_ext[1:]]

def get_reco_path(file_name):
    """
    Get the path to a file in the "raw" directory, relative
    to Django's defalt storage directory.
    @param file_name File name.
    @return the path.
    @throws Http404 if the file is not in the directory.
    """
    file_path = os.path.join("reco", file_name)
    if not default_storage.exists(file_path): 
        raise Http404 # Page not found.
    return file_path


def get_raw_path(file_name):
    """
    Get the path to a file in the "raw" directory, relative
    to Django's defalt storage directory.
    @param file_name File name.
    @return the path.
    @throws Http404 if the file is not in the directory.
    """
    file_path = os.path.join("raw", file_name)
    if not default_storage.exists(file_path): 
        raise Http404 # Page not found.
    return file_path

def is_image(file_extension):
    """
    Check if a file extension is that of an image.
    Valid image files are those with extensions supported by
    Matplotlib or PyROOT and handled by MagickWand image
    converter: at present: "ps", "eps", "png", "pdf", "gif", "jpg", 
    "jpeg". 
    @param file_extension e.g. "jpg".
    @return True if the file extension corresponds to a supported
    image, False otherwise.
    """
    return file_extension in IMAGE_TYPES

def convert_image(source_file, image_type, target_file):
    """
    Use MagickWand to convert an image file to the given type
    and save the converted image in the given location.
    @param source_file Image file path.
    @param image_type Image type e.g. "png".
    @param target_file Converted file path.
    """
    image = MagickWandImage(source_file)
    image.format = image_type
    image.save(target_file)

def thumb_image(source_file, image_type, target_file, thumb_size):
    """
    Use MagickWand to convert an image file to the given type
    and save the converted image in the given location.
    @param source_file Image file path.
    @param image_type Image type e.g. "png".
    @param target_file Converted file path.
    @param thumb_size Float with thumbnail size.
    """
    image = MagickWandImage(source_file)
    image.format = image_type
    image.thumbnail(thumb_size)
    image.save(target_file)

def create_http_response_image(image_path, image_type):
    """
    Create a Django HttpResponse and populate it with image data.
    @param image_path Path to image file.
    @param image_type Image type e.g. "png".
    @return HttpResponse with MIME type set to "image/image_type".
    """
    image_file = Image.open(image_path)
    response = HttpResponse(content_type="image/" + image_type) 
    image_file.save(response, image_type)
    return response
